==> laze-0.1.25.tar.gz: laze-0.1.25/src/model/task.rs <==
use std::collections::HashMap;
use std::path::Path;

use anyhow::{Error, Result};
use serde_yaml::Value;

use crate::nested_env;
use crate::serde_bool_helpers::{default_as_false, default_as_true};
use crate::utils::StringOrMapString;
use crate::IGNORE_SIGINT;

#[derive(Debug, Serialize, Deserialize, Eq, PartialEq)]
#[serde(deny_unknown_fields)]
pub struct Task {
    cmd: Vec<String>,
    pub help: Option<String>,
    pub required_vars: Option<Vec<String>>,
    pub export: Option<Vec<StringOrMapString>>,
    #[serde(default = "default_as_true")]
    build: bool,
    #[serde(default = "default_as_false")]
    ignore_ctrl_c: bool,
    #[serde(rename = "meta")]
    _meta: Option<Value>,
}

impl Task {
    pub fn build_app(&self) -> bool {
        self.build
    }

    pub fn execute(
        &self,
        start_dir: &Path,
        args: Option<&Vec<&str>>,
        verbose: u8,
    ) -> Result<(), Error> {
        for cmd in &self.cmd {
            use shell_words::join;
            use std::process::Command;
            let mut command = Command::new("sh");

            let cmd = cmd.replace("$$", "$");
            if verbose > 0 {
                command.arg("-x");
            }
            command.current_dir(start_dir).arg("-c");

            // handle "export:" (export laze variables to task shell environment)
            if let Some(export) = &self.export {
                for entry in export {
                    if let StringOrMapString::Map(m) = entry {
                        for (key, val) in m {
                            command.env(key, val);
                        }
                    }
                }
            }

            if let Some(args) = args {
                command.arg(cmd.clone() + " " + &join(args).to_owned());
            } else {
                command.arg(cmd);
            }

            if self.ignore_ctrl_c {
                IGNORE_SIGINT.store(true, std::sync::atomic::Ordering::SeqCst);
            }

            // run command, wait for status
            let status = command.status().expect("executing command");

            if self.ignore_ctrl_c {
                IGNORE_SIGINT.store(false, std::sync::atomic::Ordering::SeqCst);
            }

            if !status.success() {
                return Err(anyhow!("task failed"));
            }
        }
        Ok(())
    }

    fn _with_env(&self, env: &im::HashMap<&String, String>, do_eval: bool) -> Result<Task, Error> {
        Ok(Task {
            help: self.help.clone(),
            build: self.build,
            cmd: self
                .cmd
                .iter()
                .map(|cmd| {
                    if do_eval {
                        nested_env::expand_eval(cmd, env, nested_env::IfMissing::Ignore)
                    } else {
                        nested_env::expand(cmd, env, nested_env::IfMissing::Ignore)
                    }
                })
                .collect::<Result<Vec<String>, _>>()?,
            ignore_ctrl_c: self.ignore_ctrl_c,
            required_vars: self.required_vars.clone(),
            export: if do_eval {
                self.expand_export(env)
            } else {
                self.export.clone()
            },
            _meta: None,
        })
    }

    pub fn with_env(&self, env: &im::HashMap<&String, String>) -> Result<Task, Error> {
        self._with_env(env, false)
    }

    pub fn with_env_eval(&self, env: &im::HashMap<&String, String>) -> Result<Task, Error> {
        self._with_env(env, true)
    }

    fn expand_export(&self, env: &im::HashMap<&String, String>) -> Option<Vec<StringOrMapString>> {
        // TODO: yeah, this is not a beauty ...
        // what this does is, apply the env to the format as given by "export:"
        //
        // e.g., assuming `FOO=value` and FOOBAR=`other_value`:
        // ```yaml
        //
        // export:
        //   - FOO
        //   - BAR: bar
        //   - FOOBAR: ${foobar}
        // ```
        //
        // ... to export `FOO=value`, `BAR=bar` and `FOOBAR=other_value`.

        self.export.as_ref().map(|exports| {
            exports
                .iter()
                .map(|entry| match entry {
                    StringOrMapString::String(s) => StringOrMapString::Map(HashMap::from_iter([(
                        s.clone(),
                        nested_env::expand_eval(
                            format!("${{{s}}}"),
                            env,
                            nested_env::IfMissing::Empty,
                        )
                        .unwrap(),
                    )])),
                    StringOrMapString::Map(m) => {
                        StringOrMapString::Map(HashMap::from_iter(m.iter().map(|(k, v)| {
                            (
                                k.clone(),
                                nested_env::expand_eval(v, env, nested_env::IfMissing::Empty)
                                    .unwrap(),
                            )
                        })))
                    }
                })
                .collect()
        })
    }
}

==> laze-0.1.29.tar.gz: laze-0.1.29/src/model/task.rs <==
use std::path::Path;

use anyhow::{Error, Result};
use thiserror::Error;

use crate::nested_env;
use crate::serde_bool_helpers::{default_as_false, default_as_true};
use crate::IGNORE_SIGINT;

#[derive(Debug, Serialize, Deserialize, Eq, PartialEq, Clone)]
#[serde(deny_unknown_fields)]
pub struct Task {
    pub cmd: Vec<String>,
    pub help: Option<String>,
    pub required_vars: Option<Vec<String>>,
    pub required_modules: Option<Vec<String>>,
    pub export: Option<Vec<VarExportSpec>>,
    #[serde(default = "default_as_true")]
    pub build: bool,
    #[serde(default = "default_as_false")]
    pub ignore_ctrl_c: bool,
}

#[derive(Error, Debug, Serialize, Deserialize)]
pub enum TaskError {
    #[error("required variable `{var}` not set")]
    RequiredVarMissing { var: String },
    #[error("required module `{module}` not selected")]
    RequiredModuleMissing { module: String },
}

#[derive(Debug, Serialize, Deserialize, Eq, PartialEq, Clone)]
pub struct VarExportSpec {
    pub variable: String,
    pub content: Option<String>,
}

impl VarExportSpec {
    fn apply_env(&self, env: &im::HashMap<&String, String>) -> Self {
        let content = if let Some(content) = self.content.as_ref() {
            content.clone()
        } else {
            format!("${{{}}}", self.variable)
        };

        let content =
            Some(nested_env::expand_eval(content, env, nested_env::IfMissing::Empty).unwrap());

        Self {
            variable: self.variable.clone(),
            content,
        }
    }
}

impl Task {
    pub fn build_app(&self) -> bool {
        self.build
    }

    pub fn execute(
        &self,
        start_dir: &Path,
        args: Option<&Vec<&str>>,
        verbose: u8,
    ) -> Result<(), Error> {
        for cmd in &self.cmd {
            use shell_words::join;
            use std::process::Command;
            let mut command = Command::new("sh");

            let cmd = cmd.replace("$$", "$");
            if verbose > 0 {
                command.arg("-x");
            }
            command.current_dir(start_dir).arg("-c");

            // handle "export:" (export laze variables to task shell environment)
            if let Some(export) = &self.export {
                for entry in export {
                    let VarExportSpec { variable, content } = entry;
                    if let Some(val) = content {
                        command.env(variable, val);
                    }
                }
            }

            if let Some(args) = args {
                command.arg(cmd.clone() + " " + &join(args).to_owned());
            } else {
                command.arg(cmd);
            }

            if self.ignore_ctrl_c {
                IGNORE_SIGINT.store(true, std::sync::atomic::Ordering::SeqCst);
            }

            // run command, wait for status
            let status = command.status().expect("executing command");

            if self.ignore_ctrl_c {
                IGNORE_SIGINT.store(false, std::sync::atomic::Ordering::SeqCst);
            }

            if !status.success() {
                return Err(anyhow!("task failed"));
            }
        }
        Ok(())
    }

    fn _with_env(&self, env: &im::HashMap<&String, String>, do_eval: bool) -> Result<Task, Error> {
        Ok(Task {
            cmd: self
                .cmd
                .iter()
                .map(|cmd| {
                    if do_eval {
                        nested_env::expand_eval(cmd, env, nested_env::IfMissing::Ignore)
                    } else {
                        nested_env::expand(cmd, env, nested_env::IfMissing::Ignore)
                    }
                })
                .collect::<Result<Vec<String>, _>>()?,
            export: if do_eval {
                self.expand_export(env)
            } else {
                self.export.clone()
            },
            ..(*self).clone()
        })
    }

    pub fn with_env(&self, env: &im::HashMap<&String, String>) -> Result<Task, Error> {
        self._with_env(env, false)
    }

    pub fn with_env_eval(&self, env: &im::HashMap<&String, String>) -> Result<Task, Error> {
        self._with_env(env, true)
    }

    fn expand_export(&self, env: &im::HashMap<&String, String>) -> Option<Vec<VarExportSpec>> {
        // what this does is, apply the env to the format as given by "export:"
        //
        // e.g., assuming `FOO=value` and FOOBAR=`other_value`:
        // ```yaml
        //
        // export:
        //   - FOO
        //   - BAR: bar
        //   - FOOBAR: ${foobar}
        // ```
        //
        // ... to export `FOO=value`, `BAR=bar` and `FOOBAR=other_value`.

        self.export
            .as_ref()
            .map(|exports| exports.iter().map(|entry| entry.apply_env(env)).collect())
    }
}
